@@ -1,66 +1,98 @@
 
 /***************************************************************************
- * nbase_misc.c -- Some small miscelaneous utility/compatability           *
+ * nbase_misc.c -- Some small miscellaneous utility/compatibility          *
  * functions.                                                              *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2013 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
- * Foundation; Version 2 with the clarifications and exceptions described  *
- * below.  This guarantees your right to use, modify, and redistribute     *
- * this software under certain conditions.  If you wish to embed Nmap      *
- * technology into proprietary software, we sell alternative licenses      *
- * (contact sales@insecure.com).  Dozens of software vendors already       *
- * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, version detection, and the Nmap Scripting Engine.            *
+ * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
+ * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
+ * modify, and redistribute this software under certain conditions.  If    *
+ * you wish to embed Nmap technology into proprietary software, we sell    *
+ * alternative licenses (contact sales@nmap.com).  Dozens of software      *
+ * vendors already license Nmap technology such as host discovery, port    *
+ * scanning, OS detection, version detection, and the Nmap Scripting       *
+ * Engine.                                                                 *
  *                                                                         *
- * Note that the GPL places important restrictions on "derived works", yet *
- * it does not provide a detailed definition of that term.  To avoid       *
+ * Note that the GPL places important restrictions on "derivative works",  *
+ * yet it does not provide a detailed definition of that term.  To avoid   *
  * misunderstandings, we interpret that term as broadly as copyright law   *
  * allows.  For example, we consider an application to constitute a        *
- * "derivative work" for the purpose of this license if it does any of the *
- * following:                                                              *
- * o Integrates source code from Nmap                                      *
- * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-db or nmap-service-probes.                                    *
- * o Executes Nmap and parses the results (as opposed to typical shell or  *
- *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                *
- * o Integrates/includes/aggregates Nmap into a proprietary executable     *
- *   installer, such as those produced by InstallShield.                   *
- * o Links to a library or executes a program that does any of the above   *
+ * derivative work for the purpose of this license if it does any of the   *
+ * following with any software or content covered by this license          *
+ * ("Covered Software"):                                                   *
  *                                                                         *
- * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap, as well as other software we distribute under this       *
- * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
- * but is meant to clarify our interpretation of derived works with some   *
- * common examples.  Our interpretation applies only to Nmap--we don't     *
- * speak for other people's GPL works.                                     *
+ * o Integrates source code from Covered Software.                         *
  *                                                                         *
- * If you have any questions about the GPL licensing restrictions on using *
- * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
- * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
+ * or nmap-service-probes.                                                 *
  *                                                                         *
- * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * o Is designed specifically to execute Covered Software and parse the    *
+ * results (as opposed to typical shell or execution-menu apps, which will *
+ * execute anything you tell them to).                                     *
+ *                                                                         *
+ * o Includes Covered Software in a proprietary executable installer.  The *
+ * installers produced by InstallShield are an example of this.  Including *
+ * Nmap with other software in compressed or archival form does not        *
+ * trigger this provision, provided appropriate open source decompression  *
+ * or de-archiving software is widely available for no charge.  For the    *
+ * purposes of this license, an installer is considered to include Covered *
+ * Software even if it actually retrieves a copy of Covered Software from  *
+ * another source during runtime (such as by downloading it from the       *
+ * Internet).                                                              *
+ *                                                                         *
+ * o Links (statically or dynamically) to a library which does any of the  *
+ * above.                                                                  *
+ *                                                                         *
+ * o Executes a helper program, module, or script to do any of the above.  *
+ *                                                                         *
+ * This list is not exclusive, but is meant to clarify our interpretation  *
+ * of derived works with some common examples.  Other people may interpret *
+ * the plain GPL differently, so we consider this a special exception to   *
+ * the GPL that we apply to Covered Software.  Works which meet any of     *
+ * these conditions must conform to all of the terms of this license,      *
+ * particularly including the GPL Section 3 requirements of providing      *
+ * source code and allowing free redistribution of the work as a whole.    *
+ *                                                                         *
+ * As another special exception to the GPL terms, Insecure.Com LLC grants  *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
  * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two. You must obey the GNU GPL in all *
- * respects for all of the code used other than OpenSSL.  If you modify    *
- * this file, you may extend this exception to your version of the file,   *
- * but you are not obligated to do so.                                     *
+ * linked combinations including the two.                                  *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * Any redistribution of Covered Software, including any derived works,    *
+ * must obey and carry forward all of the terms of this license, including *
+ * obeying all GPL rules and restrictions.  For example, source code of    *
+ * the whole work must be provided and free redistribution must be         *
+ * allowed.  All GPL references to "this License", are to be treated as    *
+ * including the terms and conditions of this license text as well.        *
+ *                                                                         *
+ * Because this license imposes special exceptions to the GPL, Covered     *
+ * Work may not be combined (even as part of a larger work) with plain GPL *
+ * software.  The terms, conditions, and exceptions of this license must   *
+ * be included as well.  This license is incompatible with some other open *
+ * source licenses as well.  In some cases we can relicense portions of    *
+ * Nmap or grant special permissions to use it in other open source        *
+ * software.  Please contact fyodor@nmap.org with any such requests.       *
+ * Similarly, we don't incorporate incompatible open source software into  *
+ * Covered Software without special permission from the copyright holders. *
+ *                                                                         *
+ * If you have any questions about the licensing restrictions on using     *
+ * Nmap in other works, are happy to help.  As mentioned above, we also    *
+ * offer alternative license to integrate Nmap into proprietary            *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@nmap.com for further *
+ * information.                                                            *
+ *                                                                         *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
@@ -69,8 +101,8 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to nmap-dev@insecure.org for possible incorporation into the main       *
- * distribution.  By sending these changes to Fyodor or one of the         *
+ * to the dev@nmap.org mailing list for possible incorporation into the    *
+ * main distribution.  By sending these changes to Fyodor or one of the    *
  * Insecure.Org development mailing lists, or checking them into the Nmap  *
  * source code repository, it is understood (unless you specify otherwise) *
  * that you are offering the Nmap Project (Insecure.Com LLC) the           *
@@ -84,14 +116,13 @@
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License v2.0 for more details at                         *
- * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
- * included with Nmap.                                                     *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
+ * license file for more details (it's in a COPYING file included with     *
+ * Nmap, and also available from https://svn.nmap.org/nmap/COPYING         *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_misc.c 30265 2012-11-16 21:12:33Z david $ */
+/* $Id: nbase_misc.c 32741 2014-02-20 18:44:12Z dmiller $ */
 
 #include "nbase.h"
 
@@ -127,9 +158,9 @@ extern int errno;
    equivalents.  So you can use EMSGSIZE or EINTR. */
 int socket_errno() {
 #ifdef WIN32
-	return WSAGetLastError();
+    return WSAGetLastError();
 #else
-	return errno;
+    return errno;
 #endif
 }
 
@@ -140,16 +171,16 @@ int socket_errno() {
 */
 char *socket_strerror(int errnum) {
 #ifdef WIN32
-	static char buffer[128];
+    static char buffer[128];
 
-	FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM |
-		FORMAT_MESSAGE_IGNORE_INSERTS |
-		FORMAT_MESSAGE_MAX_WIDTH_MASK,
-		0, errnum, 0, buffer, sizeof(buffer), NULL);
+    FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM |
+        FORMAT_MESSAGE_IGNORE_INSERTS |
+        FORMAT_MESSAGE_MAX_WIDTH_MASK,
+        0, errnum, 0, buffer, sizeof(buffer), NULL);
 
-	return buffer;
+    return buffer;
 #else
-	return strerror(errnum);
+    return strerror(errnum);
 #endif
 }
 
@@ -191,7 +222,7 @@ int sockaddr_storage_equal(const struct sockaddr_storage *a,
 /* This function is an easier version of inet_ntop because you don't
    need to pass a dest buffer.  Instead, it returns a static buffer that
    you can use until the function is called again (by the same or another
-   thread in the process).  If there is a wierd error (like sslen being
+   thread in the process).  If there is a weird error (like sslen being
    too short) then NULL will be returned. */
 const char *inet_ntop_ez(const struct sockaddr_storage *ss, size_t sslen) {
 
@@ -207,13 +238,13 @@ const char *inet_ntop_ez(const struct sockaddr_storage *ss, size_t sslen) {
     if (sslen < sizeof(struct sockaddr_in))
       return NULL;
     return inet_ntop(AF_INET, &sin->sin_addr, str, sizeof(str));
-  } 
+  }
 #if HAVE_IPV6
   else if(sin->sin_family == AF_INET6) {
     if (sslen < sizeof(struct sockaddr_in6))
       return NULL;
     return inet_ntop(AF_INET6, &sin6->sin6_addr, str, sizeof(str));
-  } 
+  }
 #endif
   //Some laptops report the ip and address family of disabled wifi cards as null
   //so yes, we will hit this sometimes.
@@ -253,32 +284,43 @@ int dup_socket(int sd) {
 
 int unblock_socket(int sd) {
 #ifdef WIN32
-u_long one = 1;
-if(sd != 501) // Hack related to WinIP Raw Socket support
-  ioctlsocket (sd, FIONBIO, &one);
+  unsigned long one = 1;
+
+  if (sd != 501) /* Hack related to WinIP Raw Socket support */
+    ioctlsocket(sd, FIONBIO, &one);
+
+  return 0;
 #else
-int options;
-/*Unblock our socket to prevent recvfrom from blocking forever
-  on certain target ports. */
-options = O_NONBLOCK | fcntl(sd, F_GETFL);
-fcntl(sd, F_SETFL, options);
-#endif //WIN32
-return 1;
+  int options;
+
+  /* Unblock our socket to prevent recvfrom from blocking forever on certain
+   * target ports. */
+  options = fcntl(sd, F_GETFL);
+  if (options == -1)
+    return -1;
+
+  return fcntl(sd, F_SETFL, O_NONBLOCK | options);
+#endif /* WIN32 */
 }
 
 /* Convert a socket to blocking mode */
 int block_socket(int sd) {
 #ifdef WIN32
-  unsigned long options=0;
-  if(sd == 501) return 1;
-  ioctlsocket(sd, FIONBIO, &options);
+  unsigned long options = 0;
+
+  if (sd != 501)
+    ioctlsocket(sd, FIONBIO, &options);
+
+  return 0;
 #else
   int options;
-  options = (~O_NONBLOCK) & fcntl(sd, F_GETFL);
-  fcntl(sd, F_SETFL, options);
-#endif
 
-  return 1;
+  options = fcntl(sd, F_GETFL);
+  if (options == -1)
+    return -1;
+
+  return fcntl(sd, F_SETFL, (~O_NONBLOCK) & options);
+#endif
 }
 
 /* Use the SO_BINDTODEVICE sockopt to bind with a specific interface (Linux
@@ -458,6 +500,8 @@ int fselect(int s, fd_set *rmaster, fd_set *wmaster, fd_set *emaster, struct tim
         /* selecting on anything other than stdin? */
         if (s > 1)
             fds_ready = select(s, &rset, &wset, &eset, &stv);
+        else
+            usleep(stv.tv_sec * 1000000UL + stv.tv_usec);
 
         if (fds_ready > -1 && win_stdin_ready()) {
             FD_SET(STDIN_FILENO, &rset);
@@ -560,14 +604,17 @@ unsigned long nbase_crc32(unsigned char *buf, int len)
 
 /*
  * CRC-32C (Castagnoli) Cyclic Redundancy Check.
- * Taken straight from RFC 4960 (SCTP).
+ * Taken straight from Appendix C of RFC 4960 (SCTP), with the difference that
+ * the remainder register (crc32) is initialized to 0xffffffffL rather than ~0L,
+ * for correct operation on platforms where unsigned long is longer than 32
+ * bits.
  */
 
 /* Return the CRC-32C of the bytes buf[0..len-1] */
 unsigned long nbase_crc32c(unsigned char *buf, int len)
 {
   int i;
-  unsigned long crc32 = ~0L;
+  unsigned long crc32 = 0xffffffffL;
   unsigned long result;
   unsigned char byte0, byte1, byte2, byte3;
 
@@ -639,22 +686,22 @@ unsigned long nbase_adler32(unsigned char *buf, int len)
  * buffer. It uses current locale to determine if a character is printable or
  * not. It prints 73char+\n wide lines like these:
 
-0000   e8 60 65 86 d7 86 6d 30  35 97 54 87 ff 67 05 9e  .`e...m05.T..g.. 
-0010   07 5a 98 c0 ea ad 50 d2  62 4f 7b ff e1 34 f8 fc  .Z....P.bO{..4.. 
-0020   c4 84 0a 6a 39 ad 3c 10  63 b2 22 c4 24 40 f4 b1  ...j9.<.c.".$@.. 
+0000   e8 60 65 86 d7 86 6d 30  35 97 54 87 ff 67 05 9e  .`e...m05.T..g..
+0010   07 5a 98 c0 ea ad 50 d2  62 4f 7b ff e1 34 f8 fc  .Z....P.bO{..4..
+0020   c4 84 0a 6a 39 ad 3c 10  63 b2 22 c4 24 40 f4 b1  ...j9.<.c.".$@..
 
  * The lines look basically like Wireshark's hex dump.
  * WARNING: This function returns a pointer to a DYNAMICALLY allocated buffer
  * that the caller is supposed to free().
  * */
 char *hexdump(const u8 *cp, u32 length){
-  static char asciify[257];          /* Stores cha6acter table           */
+  static char asciify[257];          /* Stores character table           */
   int asc_init=0;                    /* Flag to generate table only once */
   u32 i=0, hex=0, asc=0;             /* Array indexes                    */
   u32 line_count=0;                  /* For byte count at line start     */
   char *current_line=NULL;           /* Current line to write            */
   char *buffer=NULL;                 /* Dynamic buffer we return         */
-  #define LINE_LEN 74                /* Lenght of printed line           */
+  #define LINE_LEN 74                /* Length of printed line           */
   char line2print[LINE_LEN];         /* Stores current line              */
   char printbyte[16];                /* For byte conversion              */
   int bytes2alloc;                   /* For buffer                       */
@@ -690,20 +737,20 @@ char *hexdump(const u8 *cp, u32 length){
     line2print[4]=' '; /* Replace the '\0' inserted by snprintf() with a space */
     hex=HEX_START;  asc=ASC_START;
     do { /* Print 16 bytes in both hex and ascii */
-		if (i%16 == 8) hex++; /* Insert space every 8 bytes */
+        if (i%16 == 8) hex++; /* Insert space every 8 bytes */
         snprintf(printbyte, sizeof(printbyte), "%02x", cp[i]);/* First print the hex number */
         line2print[hex++]=printbyte[0];
         line2print[hex++]=printbyte[1];
         line2print[hex++]=' ';
         line2print[asc++]=asciify[ cp[i] ]; /* Then print its ASCII equivalent */
-		i++;
-	} while (i < length && i%16 != 0);
+        i++;
+    } while (i < length && i%16 != 0);
     /* Copy line to output buffer */
     line2print[LINE_LEN-1]='\n';
     memcpy(current_line, line2print, LINE_LEN);
     current_line += LINE_LEN;
   }
-  buffer[bytes2alloc-1]='\0'; 
+  buffer[bytes2alloc-1]='\0';
   return buffer;
 } /* End of hexdump() */
 
@@ -738,13 +785,13 @@ char *format_bytecount(unsigned long long bytes, char *buf, size_t buflen) {
 
 /* Compare a canonical option name (e.g. "max-scan-delay") with a
    user-generated option such as "max_scan_delay" and returns 0 if the
-   two values are considered equivalant (for example, - and _ are
+   two values are considered equivalent (for example, - and _ are
    considered to be the same), nonzero otherwise. */
 int optcmp(const char *a, const char *b) {
   while(*a && *b) {
     if (*a == '_' || *a == '-') {
       if (*b != '_' && *b != '-')
-	return 1;
+    return 1;
     }
     else if (*a != *b)
       return 1;
@@ -759,18 +806,18 @@ int optcmp(const char *a, const char *b) {
  * is readable by the executing process.  Returns two if it is readable
  * and is a directory.  Otherwise returns 0. */
 int file_is_readable(const char *pathname) {
-	char *pathname_buf = strdup(pathname);
-	int status = 0;
-	struct stat st;
+    char *pathname_buf = strdup(pathname);
+    int status = 0;
+    struct stat st;
 
 #ifdef WIN32
-	// stat on windows only works for "dir_name" not for "dir_name/" or "dir_name\\"
-	int pathname_len = strlen(pathname_buf);
-	char last_char = pathname_buf[pathname_len - 1];
+    // stat on windows only works for "dir_name" not for "dir_name/" or "dir_name\\"
+    int pathname_len = strlen(pathname_buf);
+    char last_char = pathname_buf[pathname_len - 1];
 
-	if(	last_char == '/'
-		|| last_char == '\\')
-		pathname_buf[pathname_len - 1] = '\0';
+    if(    last_char == '/'
+        || last_char == '\\')
+        pathname_buf[pathname_len - 1] = '\0';
 
 #endif
 
