@@ -10,7 +10,7 @@
  * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we sell    *
- * alternative licenses (contact sales@insecure.com).  Dozens of software  *
+ * alternative licenses (contact sales@nmap.com).  Dozens of software      *
  * vendors already license Nmap technology such as host discovery, port    *
  * scanning, OS detection, version detection, and the Nmap Scripting       *
  * Engine.                                                                 *
@@ -66,7 +66,7 @@
  * obeying all GPL rules and restrictions.  For example, source code of    *
  * the whole work must be provided and free redistribution must be         *
  * allowed.  All GPL references to "this License", are to be treated as    *
- * including the special and conditions of the license text as well.       *
+ * including the terms and conditions of this license text as well.        *
  *                                                                         *
  * Because this license imposes special exceptions to the GPL, Covered     *
  * Work may not be combined (even as part of a larger work) with plain GPL *
@@ -84,12 +84,12 @@
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
  * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@insecure.com for     *
- * further information.                                                    *
+ * continued development of Nmap.  Please email sales@nmap.com for further *
+ * information.                                                            *
  *                                                                         *
- * If you received these files with a written license agreement or         *
- * contract stating terms other than the terms above, then that            *
- * alternative license agreement takes precedence over these comments.     *
+ * If you have received a written license agreement or contract for        *
+ * Covered Software stating terms other than these, you may choose to use  *
+ * and redistribute Covered Software under those terms instead of these.   *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
@@ -119,7 +119,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: ncat_connect.c 31563 2013-07-28 22:08:48Z fyodor $ */
+/* $Id: ncat_connect.c 32742 2014-02-20 21:22:22Z dmiller $ */
 
 #include "base64.h"
 #include "nsock.h"
@@ -298,50 +298,48 @@ static void connect_report(nsock_iod nsi)
 }
 
 /* Just like inet_socktop, but it puts IPv6 addresses in square brackets. */
-static const char *sock_to_url(const union sockaddr_u *su)
+static const char *sock_to_url(char *host_str, unsigned short port)
 {
-    static char buf[INET6_ADDRSTRLEN + 32];
-    const char *host_str;
-    unsigned short port;
+    static char buf[512];
 
-    host_str = inet_socktop(su);
-    port = inet_port(su);
-    if (su->storage.ss_family == AF_INET)
-        Snprintf(buf, sizeof(buf), "%s:%hu", host_str, port);
-    else if (su->storage.ss_family == AF_INET6)
-        Snprintf(buf, sizeof(buf), "[%s]:%hu", host_str, port);
-    else
-        bye("Unknown address family in sock_to_url_host.");
+    switch(getaddrfamily(host_str)) {
+       case -1:
+       case 1:
+           Snprintf(buf, sizeof(buf), "%s:%hu", host_str, port);
+           break;
+       case 2:
+           Snprintf(buf, sizeof(buf), "[%s]:%hu]", host_str, port);
+    }
 
     return buf;
 }
 
 static int append_connect_request_line(char **buf, size_t *size, size_t *offset,
-    const union sockaddr_u *su)
+    char* host_str,unsigned short port)
 {
     return strbuf_sprintf(buf, size, offset, "CONNECT %s HTTP/1.0\r\n",
-        sock_to_url(su));
+        sock_to_url(host_str,port));
 }
 
-static char *http_connect_request(const union sockaddr_u *su, int *n)
+static char *http_connect_request(char* host_str, unsigned short port, int *n)
 {
     char *buf = NULL;
     size_t size = 0, offset = 0;
 
-    append_connect_request_line(&buf, &size, &offset, su);
+    append_connect_request_line(&buf, &size, &offset, host_str, port);
     strbuf_append_str(&buf, &size, &offset, "\r\n");
     *n = offset;
 
     return buf;
 }
 
-static char *http_connect_request_auth(const union sockaddr_u *su, int *n,
+static char *http_connect_request_auth(char* host_str, unsigned short port, int *n,
     struct http_challenge *challenge)
 {
     char *buf = NULL;
     size_t size = 0, offset = 0;
 
-    append_connect_request_line(&buf, &size, &offset, su);
+    append_connect_request_line(&buf, &size, &offset, host_str, port);
     strbuf_append_str(&buf, &size, &offset, "Proxy-Authorization:");
     if (challenge->scheme == AUTH_BASIC) {
         char *auth_str;
@@ -364,7 +362,7 @@ static char *http_connect_request_auth(const union sockaddr_u *su, int *n,
             return NULL;
         }
         response_hdr = http_digest_proxy_authorization(challenge,
-            username, password, "CONNECT", sock_to_url(&httpconnect));
+            username, password, "CONNECT", sock_to_url(o.target,o.portno));
         if (response_hdr == NULL) {
             free(proxy_auth);
             return NULL;
@@ -405,7 +403,7 @@ static int do_proxy_http(void)
     header = NULL;
 
     /* First try a request with no authentication. */
-    request = http_connect_request(&httpconnect, &n);
+    request = http_connect_request(o.target,o.portno, &n);
     if (send(sd, request, n, 0) < 0) {
         loguser("Error sending proxy request: %s.\n", socket_strerror(socket_errno()));
         free(request);
@@ -455,7 +453,7 @@ static int do_proxy_http(void)
             goto bail;
         }
 
-        request = http_connect_request_auth(&httpconnect, &n, &challenge);
+        request = http_connect_request_auth(o.target,o.portno, &n, &challenge);
         if (request == NULL) {
             loguser("Error building Proxy-Authorization header.\n");
             http_challenge_free(&challenge);
@@ -511,12 +509,359 @@ bail:
     return -1;
 }
 
+
+/* SOCKS4a support
+ * Return a usable socket descriptor after
+ * proxy negotiation, or -1 on any error.
+ */
+static int do_proxy_socks4(void)
+{
+    struct socket_buffer stateful_buf;
+    struct socks4_data socks4msg;
+    char socksbuf[8];
+    int sd,len = 9;
+
+    sd = do_connect(SOCK_STREAM);
+    if (sd == -1) {
+        loguser("Proxy connection failed: %s.\n", socket_strerror(socket_errno()));
+        return sd;
+    }
+    socket_buffer_init(&stateful_buf, sd);
+
+    if (o.verbose) {
+        loguser("Connected to proxy %s:%hu\n", inet_socktop(&targetss),
+            inet_port(&targetss));
+    }
+
+    /* Fill the socks4_data struct */
+    zmem(&socks4msg, sizeof(socks4msg));
+    socks4msg.version = SOCKS4_VERSION;
+    socks4msg.type = SOCKS_CONNECT;
+    socks4msg.port = htons(o.portno);
+
+    switch(getaddrfamily(o.target)) {
+        case 1: // IPv4 address family
+            socks4msg.address = inet_addr(o.target);
+
+            if (o.proxy_auth){
+                memcpy(socks4msg.data, o.proxy_auth, strlen(o.proxy_auth));
+                len += strlen(o.proxy_auth);
+            }
+            break;
+
+        case 2: // IPv6 address family
+
+            loguser("Error: IPv6 addresses are not supported with Socks4.\n");
+            close(sd);
+            return -1;
+
+        case -1: // fqdn
+
+            socks4msg.address = inet_addr("0.0.0.1");
+
+            if (strlen(o.target) > SOCKS_BUFF_SIZE-2) {
+                loguser("Error: host name is too long.\n");
+                close(sd);
+                return -1;
+            }
+
+            if (o.proxy_auth){
+                if (strlen(o.target)+strlen(o.proxy_auth) > SOCKS_BUFF_SIZE-2) {
+                    loguser("Error: host name and username are too long.\n");
+                    close(sd);
+                    return -1;
+                }
+                Strncpy(socks4msg.data,o.proxy_auth,sizeof(socks4msg.data));
+                len += strlen(o.proxy_auth);
+            }
+            memcpy(socks4msg.data+(len-8), o.target, strlen(o.target));
+            len += strlen(o.target)+1;
+    }
+
+    if (send(sd, (char *) &socks4msg, len, 0) < 0) {
+        loguser("Error: sending proxy request: %s.\n", socket_strerror(socket_errno()));
+        close(sd);
+        return -1;
+    }
+
+    /* The size of the socks4 response is 8 bytes. So read exactly
+       8 bytes from the buffer */
+    if (socket_buffer_readcount(&stateful_buf, socksbuf, 8) < 0) {
+        loguser("Error: short response from proxy.\n");
+        close(sd);
+        return -1;
+    }
+
+    if (sd != -1 && socksbuf[1] != SOCKS4_CONN_ACC) {
+        loguser("Proxy connection failed.\n");
+        close(sd);
+        return -1;
+    }
+
+    return sd;
+}
+
+/* SOCKS5 support
+ * Return a usable socket descriptor after
+ * proxy negotiation, or -1 on any error.
+ */
+static int do_proxy_socks5(void)
+{
+
+    struct socket_buffer stateful_buf;
+    struct socks5_connect socks5msg;
+    uint32_t inetaddr;
+    char inet6addr[16];
+    unsigned short proxyport = htons(o.portno);
+    char socksbuf[8];
+    int sd,len,lenfqdn;
+    struct socks5_request socks5msg2;
+    struct socks5_auth socks5auth;
+    char *proxy_auth;
+    char *username;
+    char *password;
+
+    sd = do_connect(SOCK_STREAM);
+    if (sd == -1) {
+        loguser("Proxy connection failed: %s.\n", socket_strerror(socket_errno()));
+        return sd;
+    }
+
+    socket_buffer_init(&stateful_buf, sd);
+
+    if (o.verbose) {
+        loguser("Connected to proxy %s:%hu\n", inet_socktop(&targetss),
+            inet_port(&targetss));
+    }
+
+    zmem(&socks5msg,sizeof(socks5msg));
+    socks5msg.ver = SOCKS5_VERSION;
+    socks5msg.nmethods = 1;
+    socks5msg.methods[0] = SOCKS5_AUTH_NONE;
+    len = 3;
+
+    if (o.proxy_auth){
+        socks5msg.nmethods ++;
+        socks5msg.methods[1] = SOCKS5_AUTH_USERPASS;
+        len ++;
+    }
+
+    if (send(sd, (char *) &socks5msg, len, 0) < 0) {
+        loguser("Error: proxy request: %s.\n", socket_strerror(socket_errno()));
+        close(sd);
+        return -1;
+    }
+
+    /* first response just two bytes, version and auth method */
+    if (socket_buffer_readcount(&stateful_buf, socksbuf, 2) < 0) {
+        loguser("Error: malformed first response from proxy.\n");
+        close(sd);
+        return -1;
+    }
+
+    if (socksbuf[0] != 5){
+        loguser("Error: got wrong server version in response.\n");
+        close(sd);
+        return -1;
+    }
+
+    switch(socksbuf[1]) {
+        case SOCKS5_AUTH_NONE:
+            if (o.verbose)
+                loguser("No authentication needed.\n");
+            break;
+
+        case SOCKS5_AUTH_GSSAPI:
+            loguser("GSSAPI authentication method not supported.\n");
+            close(sd);
+            return -1;
+
+        case SOCKS5_AUTH_USERPASS:
+            if (o.verbose)
+                loguser("Doing username and password authentication.\n");
+
+            if(!o.proxy_auth){
+                loguser("Error: proxy requested to do authentication, but no credentials were provided.\n");
+                close(sd);
+                return -1;
+            }
+
+            if (strlen(o.proxy_auth) > SOCKS_BUFF_SIZE-2){
+                loguser("Error: username and password are too long to fit into buffer.\n");
+                close(sd);
+                return -1;
+            }
+
+            /* Split up the proxy auth argument. */
+            proxy_auth = Strdup(o.proxy_auth);
+            username = strtok(proxy_auth, ":");
+            password = strtok(NULL, ":");
+            if (password == NULL || username == NULL) {
+                free(proxy_auth);
+                loguser("Error: empty username or password.\n");
+                close(sd);
+                return -1;
+            }
+
+            /*
+             * For username/password authentication the client's authentication request is
+             * field 1: version number, 1 byte (must be 0x01 -- version of subnegotiation)
+             * field 2: username length, 1 byte
+             * field 3: username
+             * field 4: password length, 1 byte
+             * field 5: password
+             *
+             * Server response for username/password authentication:
+             * field 1: version, 1 byte
+             * field 2: status code, 1 byte.
+             * 0x00 = success
+             * any other value = failure, connection must be closed
+             */
+
+            socks5auth.ver = 1;
+            socks5auth.data[0] = strlen(username);
+            memcpy(socks5auth.data+1,username,strlen(username));
+            len = 2 + strlen(username); // (version + strlen) + username
+
+            socks5auth.data[len]=strlen(password);
+            memcpy(socks5auth.data+len,password,strlen(password));
+            len += 1 + strlen(password);
+
+            if (send(sd, (char *) &socks5auth, len, 0) < 0) {
+                loguser("Error: sending proxy authentication.\n");
+                close(sd);
+                return -1;
+            }
+
+            if (socket_buffer_readcount(&stateful_buf, socksbuf, 2) < 0) {
+                loguser("Error: malformed proxy authentication response.\n");
+                close(sd);
+                return -1;
+            }
+
+            if (socksbuf[0] != 1 || socksbuf[1] != 0) {
+                loguser("Error: authentication failed.\n");
+                close(sd);
+                return -1;
+            }
+
+            break;
+
+        default:
+            loguser("Error - can't choose any authentication method.\n");
+            close(sd);
+            return -1;
+    }
+
+    zmem(&socks5msg2,sizeof(socks5msg2));
+    socks5msg2.ver = SOCKS5_VERSION;
+    socks5msg2.cmd = SOCKS_CONNECT;
+    socks5msg2.rsv = 0;
+
+    switch(getaddrfamily(o.target)) {
+
+        case 1: // IPv4 address family
+            socks5msg2.atyp = SOCKS5_ATYP_IPv4;
+            inetaddr = inet_addr(o.target);
+            memcpy(socks5msg2.dst, &inetaddr, 4);
+            len = 4;
+            break;
+
+        case 2: // IPv6 address family
+            socks5msg2.atyp = SOCKS5_ATYP_IPv6;
+            inet_pton(AF_INET6,o.target,&inet6addr);
+            memcpy(socks5msg2.dst, inet6addr,16);
+            len = 16;
+            break;
+
+        case -1: // FQDN
+            socks5msg2.atyp = SOCKS5_ATYP_NAME;
+            lenfqdn=strlen(o.target);
+            if (lenfqdn > SOCKS_BUFF_SIZE-5){
+                loguser("Error: host name too long.\n");
+                close(sd);
+                return -1;
+            }
+            socks5msg2.dst[0]=lenfqdn;
+            memcpy(socks5msg2.dst+1,o.target,lenfqdn);
+            len = 1 + lenfqdn;
+    }
+
+    memcpy(socks5msg2.dst+len, &proxyport, sizeof(proxyport));
+    len += 2 + 1 + 3;
+
+    if (len > sizeof(socks5msg2)){
+        loguser("Error: address information too large.\n");
+        close(sd);
+        return -1;
+    }
+
+    if (send(sd, (char *) &socks5msg2, len, 0) < 0) {
+        loguser("Error: sending proxy request: %s.\n", socket_strerror(socket_errno()));
+        close(sd);
+        return -1;
+    }
+
+    /* TODO just two bytes for now, need to read more for bind */
+    if (socket_buffer_readcount(&stateful_buf, socksbuf, 2) < 0) {
+        loguser("Error: malformed second response from proxy.\n");
+        close(sd);
+        return -1;
+    }
+
+    switch(socksbuf[1]) {
+        case 0:
+            if (o.verbose)
+                loguser("connection succeeded.\n");
+            break;
+        case 1:
+            loguser("Error: general SOCKS server failure.\n");
+            close(sd);
+            return -1;
+        case 2:
+            loguser("Error: connection not allowed by ruleset.\n");
+            close(sd);
+            return -1;
+        case 3:
+            loguser("Error: Network unreachable.\n");
+            close(sd);
+            return -1;
+        case 4:
+            loguser("Error: Host unreachable.\n");
+            close(sd);
+            return -1;
+        case 5:
+            loguser("Error: Connection refused.\n");
+            close(sd);
+            return -1;
+        case 6:
+            loguser("Error: TTL expired.\n");
+            close(sd);
+            return -1;
+        case 7:
+            loguser("Error: Command not supported.\n");
+            close(sd);
+            return -1;
+        case 8:
+            loguser("Error: Address type not supported.\n");
+            close(sd);
+            return -1;
+        default:
+            loguser("Error: unassigned value in the reply.\n");
+            close(sd);
+            return -1;
+    }
+
+    return(sd);
+}
+
+
 int ncat_connect(void)
 {
     nsock_pool mypool;
     int rc;
 
-    /* Unless explicitely asked not to do so, ncat uses the
+    /* Unless explicitly asked not to do so, ncat uses the
      * fallback nsock engine to maximize compatibility between
      * operating systems and the different use cases.
      */
@@ -543,8 +888,7 @@ int ncat_connect(void)
     set_ssl_ctx_options((SSL_CTX *) nsp_ssl_init(mypool));
 #endif
 
-    if (httpconnect.storage.ss_family == AF_UNSPEC
-             && socksconnect.storage.ss_family == AF_UNSPEC) {
+    if (!o.proxytype) {
         /* A non-proxy connection. Create an iod for a new socket. */
         cs.sock_nsi = nsi_new(mypool, NULL);
         if (cs.sock_nsi == NULL)
@@ -640,65 +984,23 @@ int ncat_connect(void)
     } else {
         /* A proxy connection. */
         static int connect_socket;
-        int len;
-        char *line;
-        size_t n;
 
-        if (httpconnect.storage.ss_family != AF_UNSPEC) {
+	    if (strcmp(o.proxytype, "http") == 0) {
             connect_socket = do_proxy_http();
-            if (connect_socket == -1)
-                return 1;
-        } else if (socksconnect.storage.ss_family != AF_UNSPEC) {
-            struct socket_buffer stateful_buf;
-            struct socks4_data socks4msg;
-            char socksbuf[8];
-
-            connect_socket = do_connect(SOCK_STREAM);
-            if (connect_socket == -1) {
-                loguser("Proxy connection failed: %s.\n", socket_strerror(socket_errno()));
-                return 1;
-            }
-
-            socket_buffer_init(&stateful_buf, connect_socket);
-
-            if (o.verbose) {
-                loguser("Connected to proxy %s:%hu\n", inet_socktop(&targetss),
-                    inet_port(&targetss));
-            }
-
-            /* Fill the socks4_data struct */
-            zmem(&socks4msg, sizeof(socks4msg));
-            socks4msg.version = SOCKS4_VERSION;
-            socks4msg.type = SOCKS_CONNECT;
-            socks4msg.port = socksconnect.in.sin_port;
-            socks4msg.address = socksconnect.in.sin_addr.s_addr;
-            if (o.proxy_auth)
-                Strncpy(socks4msg.username, (char *) o.proxy_auth, sizeof(socks4msg.username));
-
-            len = 8 + strlen(socks4msg.username) + 1;
-
-            if (send(connect_socket, (char *) &socks4msg, len, 0) < 0) {
-                loguser("Error sending proxy request: %s.\n", socket_strerror(socket_errno()));
-                return 1;
-            }
-            /* The size of the socks4 response is 8 bytes. So read exactly
-               8 bytes from the buffer */
-            if (socket_buffer_readcount(&stateful_buf, socksbuf, 8) < 0) {
-                loguser("Error: short reponse from proxy.\n");
-                return 1;
-            }
-            if (socksbuf[1] != 90) {
-                loguser("Proxy connection failed.\n");
-                return 1;
-            }
-
-            /* Clear out whatever is left in the socket buffer which may be
-               already sent by proxy server along with http response headers. */
-            line = socket_buffer_remainder(&stateful_buf, &n);
-            /* Write the leftover data to stdout. */
-            Write(STDOUT_FILENO, line, n);
+        } else if (strcmp(o.proxytype, "socks4") == 0) {
+            connect_socket = do_proxy_socks4();
+        } else if (strcmp(o.proxytype, "socks5") == 0) {
+            connect_socket = do_proxy_socks5();
         }
 
+        if (connect_socket == -1)
+            return 1;
+        /* Clear out whatever is left in the socket buffer which may be
+           already sent by proxy server along with http response headers. */
+        //line = socket_buffer_remainder(&stateful_buf, &n);
+        /* Write the leftover data to stdout. */
+        //Write(STDOUT_FILENO, line, n);
+
         /* Once the proxy negotiation is done, Nsock takes control of the
            socket. */
         cs.sock_nsi = nsi_new2(mypool, connect_socket, NULL);
